FlexBar2 = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceModuleCore-2.0", "AceDebug-2.0")
FlexBar2.OO = AceLibrary("AceOO-2.0");
FlexBar2.Waterfall = AceLibrary("Waterfall-1.0");

FlexBar2:RegisterDB("FlexBar2_DB");
FlexBar2:RegisterDefaults('profile', {
	Buttons = {},
});

FlexBar2:RegisterChatCommand({"/flexbar", "/flexbar2", "/fb", "/fb2"}, 
{
    type = 'execute',
    desc = "Show configuration screen",
    func = function()
        FlexBar2.Waterfall:Open("FlexBar2");
    end
});

function FlexBar2:OnInitialize()
    self:SetDebugging(true);
	self.ModuleOnEnableList = {};
	self.ButtonList = {};
	self.ButtonMixinList = {};
	self.GroupMixinList = {};
	self.ButtonActivatorList = {};
	self.ButtonDeactivatorList = {};
	self.GroupActivatorList = {};
	self.ButtonWaterfallList = {};
	self.GroupWaterfallList = {};
	self.ButtonSettingLoaderList = {};
	self.Buttons = {};
	self:Debug("Searching for modules");
    for Name, Module in self:IterateModules() do
		self:Debug("     Found Module: " .. Name);
		if(type(Module.ButtonMixin) == "table") then
			table.insert(self.ButtonMixinList, Module.ButtonMixin);
			self:Debug("          Found ButtonMixin");
			if(type(Module.ButtonMixin.Deactivate) == "function") then
				self:Debug("               Found Activator");			
				table.insert(FlexBar2.ButtonActivatorList, Module.ButtonMixin.Activate);
			end
			if(type(Module.ButtonMixin.Deactivate) == "function") then
				self:Debug("               Found Deactivator");			
				table.insert(FlexBar2.ButtonDeactivatorList, Module.ButtonMixin.Deactivate);
			end
			if(type(Module.ButtonMixin.LoadSettings) == "function") then
				self:Debug("               Found Setting Loader");			
				table.insert(FlexBar2.ButtonSettingLoaderList, Module.ButtonMixin.LoadSettings);
			end
		end
		if(type(Module.GroupMixin) == "table") then
			self:Debug("          Found GroupMixin: ");
			self.GroupMixinList[Name] = Module.GroupMixin;
			if(type(Module.GroupMixin.Activate) == "function") then
				self:Debug("               Found Activator");
				table.insert(self.GroupActivatorList, Module.GroupMixin.Activate);
			end
		end
		if(type(Module.OnEnable) == "function") then
			self:Debug("          Found OnEnable handler");
			self.ModuleOnEnableList[Name] = Module.OnEnable;
		end
		if(type(Module.ButtonWaterfall) == "function") then
			self:Debug("          Found AceOptions");
			self.ButtonWaterfallList[Name] = Module.ButtonWaterfall;
		end
		if(type(Module.GroupWaterfall) == "function") then
			self:Debug("          Found AceOptions");
			self.GroupWaterfallList[Name] = Module.GroupWaterfall;
		end
	end
	
	self.ButtonSuperClass = self.OO.Class(unpack(self.ButtonMixinList));
		function self.ButtonSuperClass.prototype:init(Name, ...)
			FlexBar2.ButtonSuperClass.super.prototype.init(self);
			FlexBar2:Debug("          Initializing");
			if(type(Name) ~= "string") then
				FlexBar2:Debug("               Failed, invalid name");
				return false;
			end
			self.InfoTable = FlexBar2.db.profile.Buttons[Name];
			FlexBar2.Buttons[Name] = self;
			table.insert(FlexBar2.ButtonList, Name);
			for _, Func in ipairs(FlexBar2.ButtonActivatorList) do
				Func(self, ...);
			end
		end
		function self.ButtonSuperClass.prototype:LoadSettings()
			for _, Func in ipairs(FlexBar2.ButtonSettingLoaderList) do
				Func(self);
			end
		end
	self.GroupSuperClass = self.OO.Class(unpack(self.GroupMixinList));
		function self.GroupSuperClass.prototype:init(...)
			FlexBar2:Debug("          Initializing");
			FlexBar2.GroupSuperClass.super.prototype.init(self);
			for _, Func in ipairs(FlexBar2.GroupActivatorList) do
				Func(self, ...);
			end
		end
end

function FlexBar2:Create(Name)
	if(type(FlexBar2.db.profile.Buttons[Name]) ~= "table") then
		FlexBar2.db.profile.Buttons[Name] = { Generate = true};
		FlexBar2:Generate(Name);
		return true;
	else
		FlexBar2:Print("Button already exists");
		return false;
	end
end


function FlexBar2:Delete(Name)
	if(type(FlexBar2.db.profile.Buttons[Name]) == "table") then
		FlexBar2.db.profile.Buttons[Name] = nil;
		local Button = FlexBar2.Buttons[Name]
		for _, Func in ipairs(FlexBar2.ButtonDeactivatorList) do
			Func(Button);
		end
		FlexBar2.Buttons[Name] = nil;
		return true;
	else
		FlexBar2:Print("Cannot delete non-existant button");
		return false;
	end
	
end

function FlexBar2:Generate(Name)
	self:Debug(Name);
	if(self.Buttons[Name] ~= nil) then
		self:Debug("          Error: Button already exists, aborting");
		return false;
	end
	self.ButtonSuperClass:new(Name);
	return true;
end

function FlexBar2:ToButtonTable(ButtonRange)
	if(type(ButtonRange) == "string") then
		if(ButtonRange:find("(%d+)-(%d+)")) then
			local ButtonTable = {};
			local First, Last = ButtonRange:find("(%d+)-(%d+)");
			for ButtonNumber = First, Last do
				ButtonTable:insert(tostring(ButtonNumber));
			end
		elseif(ButtonRange:find(";")) then
			ButtonTable = { string.split(";", ButtonRange) };
		else
			ButtonTable = { ButtonRange };
		end
		for _, ButtonName in ipairs(ButtonTable) do
			if(not ButtonName:find("^[%w_]+$")) then
				return false;
			end
		end
		return ButtonTable;
	else
		return false;
	end
end

function FlexBar2:GetSetting(ButtonTable, Setting)
	if(type(ButtonTable) ~= "table" or type(Setting) ~= "string") then return nil; end
	return FlexBar2.db.profile.Buttons[ButtonTable[1]][Setting];
end

function FlexBar2:ApplySetting(ButtonTable, Setting, Value)
	if(type(ButtonTable) ~= "table" or type(Setting) ~= "string") then return nil; end
	for _, ButtonName in ipairs(ButtonTable) do
		FlexBar2.db.profile.Buttons[ButtonName][Setting] = Value;
	end
end

function FlexBar2:OnEnable()
	local Waterfall = self.Waterfall;
	self:Debug("Setting up Waterfall");
	local WaterfallTree = 
	{
		{
			text = "General Settings",
			id = "gnrlset",
			isOpen = true,
		},
		{
			text = "Button Control",
			id = "buttonctrl",
			isOpen = false,
		},
		{
			text = "Group Control",
			id = "groupctrl",
			isOpen = false,
		},
	};
	local buttonctrl_CreateTemp, buttonctrl_EditSave, buttonctrl_EditTemp;
	local function BuildWaterfallTree(Id)
		if not Id then return ""; end
		if(Id == "gnrlset") then			
		elseif(Id == "buttonctrl") then
			if(not buttonctrl_EditSave or type(FlexBar2.Buttons[buttonctrl_EditSave]) ~= "table") then
				if(type(FlexBar2.Buttons[buttonctrl_EditSave]) ~= "table" and type(buttonctrl_EditSave) == "string") then
					FlexBar2:Print("Non-existant button");
					buttonctrl_EditSave = nil;
				end
				FlexBar2:Debug("Waterfall: ButtonCtrl main");
				-- Create Button --
				Waterfall:AddControl(
				"type", "label",
				"text", "Create Button: ");
				
				Waterfall:AddControl(
				"type","textbox",
				"width",200,
				"fullRefresh", true,
				"noNewLine", true,
				"setFunc", function() FlexBar2:Create(buttonctrl_CreateTemp); buttonctrl_EditSave = buttonctrl_CreateTemp; end,
				"getFunc", function() return ""; end,
				"changedFunc", function(ButtonName) buttonctrl_CreateTemp = ButtonName; end);
				
				Waterfall:AddControl(
				"type","button",
				"text","Create",
				"fullRefresh", true,
				"noNewLine",true,
				"width",80,
				"causeRefresh",true,
				"execFunc", function() FlexBar2:Create(buttonctrl_CreateTemp); buttonctrl_EditSave = buttonctrl_CreateTemp; end);
				
				-- Edit Button --
				Waterfall:AddControl(
				"type", "label",
				"text", "Edit Button: ");
				
				Waterfall:AddControl(
				"type","textbox",
				"width",200,
				"noNewLine", true,
				"fullRefresh", true,
				"setFunc", function(ButtonName) buttonctrl_EditSave = ButtonName end,
				"changedFunc", function(ButtonName) buttonctrl_EditTemp = ButtonName end);
				
				Waterfall:AddControl(
				"type","button",
				"text","Edit",
				"noNewLine",true,
				"fullRefresh", true,
				"width",80,
				"execFunc", function() buttonctrl_EditSave = buttonctrl_EditTemp end);
				return "Basic Button Control";
			else
				FlexBar2:Debug("Waterfall: Buttonctrl edit");
				for Module, Data in pairs(FlexBar2.ButtonWaterfallList) do
					if(type(Data) == "function") then
						Data("edit", buttonctrl_EditSave);
					end
				end
				-- Delete Button --
				Waterfall:AddControl(
				"type", "button",
				"text","Delete Button",
				"fullRefresh", true,
				"width", 100,
				"execFunc", function() FlexBar2:Delete(buttonctrl_EditSave); buttonctrl_EditSave = nil; end);
				
				-- Cancel Button --
				Waterfall:AddControl(
				"type", "button",
				"text","Cancel changes",
				"fullRefresh", true,
				"noNewLine", true,
				"width", 130,
				"execFunc", function()
					buttonctrl_EditSave = nil;
					end,
				"execArg1");
			end

				-- Save Button --
				Waterfall:AddControl(
				"type", "button",
				"text","Save changes",
				"fullRefresh", true,
				"noNewLine", true,
				"width", 130,
				"execFunc", function()
					
					for Module, Data in pairs(FlexBar2.ButtonWaterfallList) do
						if(type(Data) == "function") then
							Data("save", buttonctrl_EditSave, FlexBar2:ToButtonTable(buttonctrl_EditSave));
						end
					end
					local ButtonTable = FlexBar2:ToButtonTable(buttonctrl_EditSave);
					for _, ButtonName in ipairs(ButtonTable) do
						FlexBar2.Buttons[ButtonName]:LoadSettings();
					end
					buttonctrl_EditSave = nil;
				end);

				return "Editing button(s) settings: " .. buttonctrl_EditSave;
		elseif(Id == "groupctrl") then
			
		end
	end
	Waterfall:Register('FlexBar2',
	'title', 'FlexBar2 Config',
	'width', 800,
	'height', 800,
	'tree', WaterfallTree,
	'children', BuildWaterfallTree);
	
	self:Debug("Generating buttons");
	for Name, InfoTable in pairs(self.db.profile.Buttons) do
		if(InfoTable.Generate == true) then
			self:Debug("     Generating " .. Name);
			self:Generate(Name);
		end
	end
	for _, Func in ipairs(self.ModuleOnEnableList) do
		Func(self);
	end
end

function FlexBar2:OnDisable()
    
end

function FlexBar2:OnModuleCreated(Name, ref)

end
